@@ -1,9 +1,10 @@
 from pyrf.config import SweepEntry, TriggerSettings, TRIGGER_TYPE_LEVEL, TRIGGER_TYPE_NONE, TriggerSettingsError
 from pyrf.connectors.blocking import PlainSocketConnector
 from pyrf.connectors.base import sync_async
-from pyrf.vrt import vrt_packet_reader, I_ONLY, IQ
-from pyrf.windows_util import get_broadcast_addresses
-from pyrf.units import M
+from pyrf.vrt import vrt_packet_reader
+from pyrf import windows_util
+from pyrf import linux_util
+from pyrf.devices.thinkrf_properties import wsa_properties
 
 import struct
 import socket
@@ -17,185 +18,6 @@ _DISCOVERY_QUERY_FORMAT = '>LL'
 DISCOVERY_QUERY = struct.pack(_DISCOVERY_QUERY_FORMAT,
     _DISCOVERY_QUERY_CODE, _DISCOVERY_QUERY_VERSION)
 
-class WSA4000Properties(object):
-    model = 'WSA4000'
-
-    ADC_DYNAMIC_RANGE = 72.5
-    NOISEFLOOR_CALIBRATION = -10
-    CAPTURE_FREQ_RANGES = [(0, 40*M, I_ONLY), (90*M, 10000*M, IQ)]
-    SWEEP_FREQ_RANGE = (90*M, 10000*M)
-
-    RFE_MODES = ('ZIF',)
-
-    DEFAULT_SAMPLE_TYPE = {'ZIF': IQ} # almost true, see CAPTURE_FREQ_RANGES
-    FULL_BW = {'ZIF': 125*M}
-    USABLE_BW = {'ZIF': 90*M}
-    MIN_TUNABLE = {'ZIF': 90*M}
-    MAX_TUNABLE = {'ZIF': 10000*M}
-    MIN_DECIMATION = {'ZIF': 4}
-    MAX_DECIMATION = {'ZIF': 1023}
-    DECIMATED_USABLE = 0.5
-    PASS_BAND_CENTER = {'ZIF': 0.5}
-    DC_OFFSET_BW = 240000 # XXX: an educated guess
-    TUNING_RESOLUTION = 100000
-    FSHIFT_AVAILABLE = {'ZIF': True}
-    SWEEP_SETTINGS = ['fstart', 'fstop', 'fstep', 'fshift', 'decimation',
-        'antenna', 'gain', 'ifgain', 'spp', 'ppb', 'dwell_s', 'dwell_us',
-        'trigtype', 'level_fstart', 'level_fstop', 'level_amplitude']
-
-    SPECA_DEFAULTS = {
-        'mode': 'ZIF',
-        'center': 2450 * M,
-        'rbw': 122070,
-        'span': 125 * M,
-        'decimation': 1,
-        'fshift': 0,
-        'device_settings': {
-            'antenna': 1,
-            'ifgain': 0,
-            'gain': 'low',
-            },
-        'device_class': 'thinkrf.WSA',
-        'device_identifier': 'unknown',
-        }
-
-
-class WSA5000_220Properties(object):
-    model = 'WSA5000-220'
-    MINIMUM_FW_VERSION = '3.2.0-rc1'
-
-    ADC_DYNAMIC_RANGE = 72.5
-    NOISEFLOOR_CALIBRATION = -10
-    CAPTURE_FREQ_RANGES = [(50*M, 20000*M, IQ)]
-    SWEEP_FREQ_RANGE = (100*M, 20000*M)
-    RFE_ATTENUATION = 20
-    RFE_MODES = ('ZIF', 'SH', 'SHN', 'HDR', 'IQIN', 'DD')
-    DEFAULT_SAMPLE_TYPE = {
-        'ZIF': IQ,
-        'SH': I_ONLY,
-        'SHN': I_ONLY,
-        'HDR': I_ONLY,
-        'IQIN': IQ,
-        'DD': I_ONLY,
-        }
-    FULL_BW = {
-        'ZIF': 125 * M,
-        'HDR': 0.16276 * M,
-        'SH': 62.5 * M,
-        'SHN': 62.5 * M,
-        'IQIN': 125 * M,
-        'DD': 62.5 * M,
-        }
-    USABLE_BW = {
-        'ZIF': 100 * M,
-        'HDR': 0.1 * M,
-        'SH': 40 * M,
-        'SHN': 10 * M,
-        'IQIN': 100 * M,
-        'DD': 62.5 * M,
-        }
-    MIN_TUNABLE = {
-        'ZIF': 50 * M,
-        'HDR': 50 * M,
-        'SH': 50 * M,
-        'SHN': 50 * M,
-        'IQIN': 0,
-        'DD': 31.24 * M,
-        }
-    MAX_TUNABLE = {
-        'ZIF': 20000 * M,
-        'HDR': 20000 * M,
-        'SH': 20000 * M,
-        'SHN': 20000 * M,
-        'IQIN': 0,
-        'DD': 31.24 * M,
-        }
-    MIN_DECIMATION = {
-        'ZIF': 4,
-        'HDR': None,
-        'SH': 4,
-        'SHN': 4,
-        'IQIN': 4,
-        'DD': 4,
-        }
-    MAX_DECIMATION = {
-        'ZIF': 1024,
-        'HDR': None,
-        'SH': 4,
-        'SHN': 4,
-        'IQIN': 1024,
-        'DD': 1024,
-        }
-    DECIMATED_USABLE = 0.80
-    PASS_BAND_CENTER = {
-        'ZIF': 0.5,
-        'HDR': 0.6,
-        'SH': 0.56,
-        'SHN': 0.56,
-        'IQIN': 0.5,
-        'DD': 0.5,
-        }
-    DC_OFFSET_BW = 240000 # XXX: an educated guess
-    TUNING_RESOLUTION = 100000
-    FSHIFT_AVAILABLE = {
-        'ZIF': True,
-        'HDR': False,
-        'SH': True,
-        'SHN': True,
-        'IQIN': True,
-        'DD': True,
-        }
-    MAX_FSHIFT = {'ZIF': 62.5*M}
-    SWEEP_SETTINGS = ['rfe_mode', 'fstart', 'fstop', 'fstep', 'fshift',
-        'decimation', 'attenuator', 'ifgain', 'spp', 'ppb',
-        'dwell_s', 'dwell_us',
-        'trigtype', 'level_fstart', 'level_fstop', 'level_amplitude']
-
-    LEVEL_TRIGGER_RFE_MODES = ['SH', 'SHN', 'ZIF']
-
-    SPECA_DEFAULTS = {
-        'mode': 'ZIF',
-        'center': 2450 * M,
-        'rbw': 122070,
-        'span': 125 * M,
-        'decimation': 1,
-        'fshift': 0,
-        'device_settings': {
-            'attenuator': True,
-            'iq_output_path': 'DIGITIZER',
-            'pll_reference': 'INT',
-            'trigger': {'type': 'NONE',
-                        'fstart': 2440 * M,
-                        'fstop': 2460 * M,
-                        'amplitude': -110},
-            },
-        'device_class': 'thinkrf.WSA',
-        'device_identifier': 'unknown',
-        }
-
-
-class WSA5000_220_v2Properties(WSA5000_220Properties):
-    model = 'WSA5000-220 v2'
-    # v2 -> hardware revision without SHN mode
-    RFE_MODES = ('ZIF', 'SH', 'HDR', 'IQIN', 'DD')
-
-
-class WSA5000_208Properties(WSA5000_220Properties):
-    model = 'WSA5000-208'
-    # 208 -> limited to 8GHz
-
-    MAX_TUNABLE = dict((mode, min(8000*M, f))
-        for mode, f in WSA5000_220Properties.MAX_TUNABLE.iteritems())
-
-class WSA5000_108Properties(WSA5000_208Properties):
-    model = 'WSA5000-108'
-    # 108 -> limited to SHN, HDR, and DD mode
-    RFE_MODES = ('SHN', 'HDR', 'DD')
-
-class WSA5000_208_v2Properties(WSA5000_220_v2Properties, WSA5000_208Properties):
-    model = 'WSA5000-208 v2'
-
-
 class WSA(object):
     """
     Interface for WSA4000 and WSA5000
@@ -226,13 +48,30 @@ class WSA(object):
         self.connector = connector
         self._output_file = None
 
-    def set_capture_output(self, output_file=None):
+    def async_connector(self):
+        """
+        Return True if the connector being used is asynchronous
+        """
+        return hasattr(self.connector, 'vrt_callback')
+
+    def set_async_callback(self, callback):
+        self.connector.vrt_callback = callback
+
+    def set_recording_output(self, output_file=None):
         """
         Dump a recording of all the received packets to output_file
         """
         self.connector.set_recording_output(output_file)
         self._output_file = output_file
 
+    def inject_recording_state(self, state):
+        """
+        Inject the current speca state into the recording stream when
+        the next capture is received.  Replaces previous data if not
+        yet sent.
+        """
+        self.connector.inject_recording_state(state)
+
     @sync_async
     def connect(self, host):
         """
@@ -241,21 +80,10 @@ class WSA(object):
         :param host: the hostname or IP to connect to
         """
         yield self.connector.connect(host)
-        device_id = (yield self.scpiget(":*idn?"))
-        if device_id.startswith('ThinkRF,WSA4000'):
-            self.properties = WSA4000Properties
-        elif device_id.startswith('ThinkRF,WSA5000-220 v2'):
-            self.properties = WSA5000_220_v2Properties
-        elif device_id.startswith('ThinkRF,WSA5000-208 v2'):
-            self.properties = WSA5000_208_v2Properties
-        elif device_id.startswith('ThinkRF,WSA5000-208'):
-            self.properties = WSA5000_208Properties
-        elif device_id.startswith('ThinkRF,WSA5000-108'):
-            self.properties = WSA5000_108Properties
-        else:
-            self.properties = WSA5000_220Properties
+        self.device_id = (yield self.scpiget(":*idn?"))
+        self.properties = wsa_properties(self.device_id)
 
-        self.fw_version = device_id.split(',')[-1]
+        self.fw_version = self.device_id.split(',')[-1]
         self.device_state = {}
 
     def disconnect(self):
@@ -608,7 +436,7 @@ class WSA(object):
     @sync_async
     def request_read_perm(self):
         """
-        Aquire exclusive permission to read data from the WSA.
+        Acquire exclusive permission to read data from the WSA.
 
         :returns: True if allowed to read, False if not
         """
@@ -909,7 +737,9 @@ def discover_wsa(wait_time=0.125):
     wsa_list = []
 
     if platform.system() == 'Windows':
-        destinations = get_broadcast_addresses()
+        destinations = windows_util.get_broadcast_addresses()
+    elif platform.system() == 'Linux':
+        destinations = linux_util.get_broadcast_addresses()
     else:
         destinations = ['<broadcast>']
 
