@@ -9,12 +9,10 @@ All the buttons and controls and their callback functions are built in
 and placed to left of the controls.
 """
 
-import sys
 from PySide import QtGui, QtCore
 import numpy as np
 import math
 
-from contextlib import contextmanager
 from pkg_resources import parse_version
 
 from pyrf.gui import colors
@@ -29,18 +27,33 @@ from pyrf.devices.thinkrf import WSA
 from pyrf.vrt import (I_ONLY, VRT_IFDATA_I14Q14, VRT_IFDATA_I14,
     VRT_IFDATA_I24, VRT_IFDATA_PSD8)
 
-from util import find_max_index, find_nearest_index
-from util import hotkey_util, update_marker_traces
-import control_util as cu
-from plot_widget import Plot
-from device_controls import DeviceControls
-from frequency_controls import FrequencyControls
-from discovery_widget import DiscoveryWidget
-from trace_controls import TraceControls
+from pyrf.gui.util import find_nearest_index
+from pyrf.gui.plot_widget import Plot
+from pyrf.gui.device_controls import DeviceControls
+from pyrf.gui.frequency_controls import FrequencyControls
+from pyrf.gui.discovery_widget import DiscoveryWidget
+from pyrf.gui.trace_controls import TraceControls
 
-PLOT_YMIN = -140
-PLOT_YMAX = 0
+VIEW_OPTIONS = [
+    ('&IQ Plots', 'iq_plots', False),
+    ('&Waterfall Plot', 'waterfall_plot', False),
+    ]
 
+DSP_OPTIONS = [
+    ('&IQ Offset Correction', 'dsp.correct_phase', True),
+    ('&DC Offset', 'dsp.hide_differential_dc_offset', True),
+    ('&Convert to dBm', 'dsp.convert_to_dbm', True),
+    ('Apply &Spectral Inversion', 'dsp.apply_spec_inv', True),
+    ('Apply &Hanning Window', 'dsp.apply_window', True),
+    ]
+
+DEVELOPER_OPTIONS = [
+    ('Show &Attenuated Edges', 'show_attenuated_edges', False),
+    ('Show &Sweep Steps', 'show_sweep_steps', False),
+    ('&Free Plot Adjustment', 'free_plot_adjustment', False),
+    ]
+
+# FIXME: we shouldn't be calculating fft in this module
 ZIF_BITS = 2**13
 CONST_POINTS = 512
 
@@ -48,6 +61,9 @@ CONST_POINTS = 512
 IQ_PLOT_YMIN = {'ZIF': -1, 'HDR': -1, 'SH': -1, 'SHN': -1, 'IQIN': -1, 'DD': -1}
 IQ_PLOT_YMAX = {'ZIF': 1, 'HDR': 1, 'SH': -1, 'SHN': -1, 'IQIN': 1, 'DD': 1}
 
+MINIMUM_WIDTH = 600
+MINIMUM_HEIGHT = 600
+
 try:
     from twisted.internet.defer import inlineCallbacks
 except ImportError:
@@ -59,43 +75,88 @@ class MainWindow(QtGui.QMainWindow):
     """
     The main window and menus
     """
-    def __init__(self, output_file=None):
+    def __init__(self, dut_address=None, playback_filename=None):
         super(MainWindow, self).__init__()
         screen = QtGui.QDesktopWidget().screenGeometry()
-        WINDOW_WIDTH = screen.width() * 0.7
-        WINDOW_HEIGHT = screen.height() * 0.6
+        WINDOW_WIDTH = max(screen.width() * 0.7, MINIMUM_WIDTH)
+        WINDOW_HEIGHT = max(screen.height() * 0.6, MINIMUM_HEIGHT)
         self.resize(WINDOW_WIDTH,WINDOW_HEIGHT)
 
-        self.init_menu_bar()
         self.controller = SpecAController()
-        self.initUI()
+        self.init_menu_bar()
+        self.initUI(dut_address, playback_filename)
 
-    def initUI(self):
-        name = None
-        if len(sys.argv) > 1:
-            name = sys.argv[1]
+    def initUI(self, dut_address, playback_filename):
         self.mainPanel = MainPanel(self.controller, self)
 
-        self.setWindowTitle('Spectrum Analyzer')
+        self.setWindowTitle('PyRF RTSA')
         self.setCentralWidget(self.mainPanel)
-        if name:
-            self.open_device(name, True)
+        if dut_address:
+            self.open_device(dut_address, True)
+        elif playback_filename:
+            self.start_playback(playback_filename)
         else:
             self.open_device_dialog()
 
     def init_menu_bar(self):
-        openAction = QtGui.QAction('&Open Device', self)
-        openAction.triggered.connect(self.open_device_dialog)
-        exitAction = QtGui.QAction('&Exit', self)
-        exitAction.setShortcut('Ctrl+Q')
-        exitAction.triggered.connect(self.close)
+        open_action = QtGui.QAction('&Open Device', self)
+        open_action.triggered.connect(self.open_device_dialog)
+        play_action = QtGui.QAction('&Playback Recording', self)
+        play_action.triggered.connect(self.open_playback_dialog)
+        self.record_action = QtGui.QAction('Start &Recording', self)
+        self.record_action.triggered.connect(self.start_recording)
+        self.record_action.setDisabled(True)
+        self.stop_action = QtGui.QAction('&Stop Recording', self)
+        self.stop_action.triggered.connect(self.stop_recording)
+        self.stop_action.setDisabled(True)
+        exit_action = QtGui.QAction('&Exit', self)
+        exit_action.setShortcut('Ctrl+Q')
+        exit_action.triggered.connect(self.close)
         menubar = self.menuBar()
+        file_menu = menubar.addMenu('&File')
+        file_menu.addAction(open_action)
+        file_menu.addAction(play_action)
+        file_menu.addSeparator()
+        file_menu.addAction(self.record_action)
+        file_menu.addAction(self.stop_action)
+        file_menu.addSeparator()
+        file_menu.addAction(exit_action)
 
-        fileMenu = menubar.addMenu('&File')
-        fileMenu.addAction(openAction)
-        fileMenu.addAction(exitAction)
+        def checkbox_action(apply_fn, text, option, default):
+            action = QtGui.QAction(text, self)
+            action.setCheckable(True)
+            if default:
+                action.toggle()
+            action.triggered.connect(lambda:
+                apply_fn(**{option: action.isChecked()}))
+            return action
+
+        self.view_menu = menubar.addMenu('&View')
+        for text, option, default in VIEW_OPTIONS:
+            self.view_menu.addAction(checkbox_action(
+                self.controller.apply_options, text, option, default))
 
         self.dsp_menu = menubar.addMenu('&DSP Options')
+        for text, option, default in DSP_OPTIONS:
+            self.dsp_menu.addAction(checkbox_action(
+                self.controller.apply_options, text, option, default))
+
+        self.developer_menu = menubar.addMenu('D&eveloper Options')
+        for text, option, default in DEVELOPER_OPTIONS:
+            self.developer_menu.addAction(checkbox_action(
+                self.controller.apply_options, text, option, default))
+
+        self.controller.apply_options(
+            **dict((option, default) for text, option, default
+            in VIEW_OPTIONS + DSP_OPTIONS + DEVELOPER_OPTIONS))
+
+    def start_recording(self):
+        self.stop_action.setDisabled(False)
+        self.controller.start_recording()
+
+    def stop_recording(self):
+        self.stop_action.setDisabled(True)
+        self.controller.stop_recording()
 
     def open_device_dialog(self):
         self.discovery_widget = DiscoveryWidget(
@@ -103,6 +164,19 @@ class MainWindow(QtGui.QMainWindow):
             name="Open Device")
         self.discovery_widget.show()
 
+    def open_playback_dialog(self):
+        self.controller.set_device(None)
+        playback_filename, file_type = QtGui.QFileDialog.getOpenFileName(self,
+            "Play Recording", None, "VRT Packet Capture Files (*.vrt)")
+        if playback_filename:
+            self.start_playback(playback_filename)
+
+    def start_playback(self, playback_filename):
+        self.record_action.setDisabled(True)
+        self.stop_action.setDisabled(True)
+        self.controller.set_device(playback_filename=playback_filename)
+        self.show()
+
     @inlineCallbacks
     def open_device(self, name, ok):
         if not ok:
@@ -112,7 +186,7 @@ class MainWindow(QtGui.QMainWindow):
         self.show()
         dut = WSA(connector=TwistedConnector(self._get_reactor()))
         yield dut.connect(name)
-        self.setWindowTitle('Spectrum Analyzer Connected To: %s' %name)
+        self.setWindowTitle('PyRF RTSA Connected To: %s' %name)
         if hasattr(dut.properties, 'MINIMUM_FW_VERSION') and parse_version(
                 dut.fw_version) < parse_version(dut.properties.MINIMUM_FW_VERSION):
             too_old = QtGui.QMessageBox()
@@ -122,14 +196,13 @@ class MainWindow(QtGui.QMainWindow):
                 dut.fw_version, dut.properties.MINIMUM_FW_VERSION))
             too_old.exec_()
         self.controller.set_device(dut)
-
+        self.record_action.setDisabled(False)
+        self.stop_action.setDisabled(True)
 
     def closeEvent(self, event):
-        if self.mainPanel.dut:
-            self.mainPanel.dut.abort()
-            self.mainPanel.dut.flush()
-            self.mainPanel.dut.reset()
         event.accept()
+        self.controller.stop_recording()
+        self.controller.set_device()
         self._get_reactor().stop()
 
     def _get_reactor(self):
@@ -137,7 +210,6 @@ class MainWindow(QtGui.QMainWindow):
         from twisted.internet import reactor
         return reactor
 
-
 class MainPanel(QtGui.QWidget):
     """
     The spectrum view and controls
@@ -147,36 +219,40 @@ class MainPanel(QtGui.QWidget):
         controller.device_change.connect(self.device_changed)
         controller.state_change.connect(self.state_changed)
         controller.capture_receive.connect(self.capture_received)
+        controller.options_change.connect(self.options_changed)
 
         self._main_window = main_window
-        self._dsp_menu = self._main_window.dsp_menu
 
         self.ref_level = 0
         self.dut = None
         self.control_widgets = []
         super(MainPanel, self).__init__()
         screen = QtGui.QDesktopWidget().screenGeometry()
-        self.setMinimumWidth(screen.width() * 0.7)
-        self.setMinimumHeight(screen.height() * 0.6)
+        self.setMinimumWidth(MINIMUM_WIDTH)
+        self.setMinimumHeight(MINIMUM_HEIGHT)
         self.plot_state = None
+        self.gui_state = None
         # plot window
         self._plot = Plot(controller, self)
-        self._marker_trace = None
+        self._plot.user_xrange_change.connect(controller.user_xrange_changed)
+
         self._vrt_context = {}
         self.initUI()
         self.disable_controls()
         self.ref_level = 0
         self.plot_state = None
 
-        self.freq_range = None, None
+        self._waterfall_range = None, None, None
+
+        self.options_changed(controller.get_options(),
+            ['iq_plots', 'waterfall_plot'])
 
     def device_changed(self, dut):
         self.plot_state = gui_config.PlotState(dut.properties)
+        self.trace_group.plot_state = self.plot_state
         self.dut_prop = dut.properties
 
         self.enable_controls()
-        self._plot.const_window.show()
-        self._plot.iq_window.show()
 
     def state_changed(self, state, changed):
         """
@@ -187,34 +263,27 @@ class MainPanel(QtGui.QWidget):
         self.gui_state = state
 
         if 'mode' in changed:
-            self.rfe_mode = state.rfe_mode()  # used by recentering code
-            if state.sweeping():
-                self._plot.const_window.hide()
-                self._plot.iq_window.hide()
-                return
+            rfe_mode = state.rfe_mode()
+            self._update_iq_plot_visibility()
 
-            self._plot.const_window.show()
-            self._plot.iq_window.show()
-
-            if self.rfe_mode in ('DD', 'IQIN'):
-                freq = self.dut_prop.MIN_TUNABLE[self.rfe_mode]
-                full_bw = self.dut_prop.FULL_BW[self.rfe_mode]
+            if rfe_mode in ('DD', 'IQIN'):
+                freq = self.dut_prop.MIN_TUNABLE[rfe_mode]
+                full_bw = self.dut_prop.FULL_BW[rfe_mode]
 
                 self._plot.center_view(freq, full_bw, self.plot_state.min_level, self.plot_state.ref_level)
-                self._plot.iq_window.setYRange(IQ_PLOT_YMIN[self.rfe_mode],
-                                        IQ_PLOT_YMAX[self.rfe_mode])
+                self._plot.iq_window.setYRange(IQ_PLOT_YMIN[rfe_mode],
+                                        IQ_PLOT_YMAX[rfe_mode])
             else:
                 freq = state.center
                 full_bw = state.span
 
-                self._plot.center_view(freq - full_bw/2, freq + full_bw/2, self.plot_state.min_level, self.plot_state.ref_level)
-                self._plot.iq_window.setYRange(IQ_PLOT_YMIN[self.rfe_mode],
-                                        IQ_PLOT_YMAX[self.rfe_mode])
+                self._plot.center_view(freq - full_bw/2, freq + full_bw/2)
+                self._plot.iq_window.setYRange(IQ_PLOT_YMIN[rfe_mode],
+                                        IQ_PLOT_YMAX[rfe_mode])
         if 'device_settings.iq_output_path' in changed:
-            if 'CONNECTOR' in state.device_settings['iq_output_path']:
+            if state.device_settings['iq_output_path'] == 'CONNECTOR':
                 # remove plots
                 self._plot_group.hide()
-                self._trace_group.hide()
                 self._plot_layout.hide()
                 if self._main_window.isMaximized():
                     self._main_window.showNormal()
@@ -231,41 +300,70 @@ class MainPanel(QtGui.QWidget):
                 self.resize(0,0)
                 self._main_window.resize(0,0)
 
-            elif 'DIGITIZER' in state.device_settings['iq_output_path']:
+            else:
                 # show plots
                 self._plot_group.show()
-                self._trace_group.show()
                 self._plot_layout.show()
 
                 # resize window
                 for x in range(self.plot_width):
                     self._grid.setColumnMinimumWidth(x, 300)
                 screen = QtGui.QDesktopWidget().screenGeometry()
-                self.setMinimumWidth(screen.width() * 0.7)
-                self.setMinimumHeight(screen.height() * 0.6)
+                self.setMinimumWidth(MINIMUM_WIDTH)
+                self.setMinimumHeight(MINIMUM_HEIGHT)
+                WINDOW_WIDTH = max(screen.width() * 0.7, MINIMUM_WIDTH)
+                WINDOW_HEIGHT = max(screen.height() * 0.6, MINIMUM_HEIGHT)
+                self.resize(WINDOW_WIDTH,WINDOW_HEIGHT)
+
 
     def keyPressEvent(self, event):
-        if self.dut_prop:
-            hotkey_util(self, event)
+        if not self.dut_prop:
+            return
+
+        hotkey_dict = {
+            'M': self.trace_group._marker_control,
+            'P': self.trace_group._find_peak,
+            }
+
+        arrow_dict = {
+            '32': 'SPACE',
+            '16777235': 'UP KEY',
+            '16777237': 'DOWN KEY',
+            '16777234': 'LEFT KEY',
+            '16777236': 'RIGHT KEY',
+            }
+
+        if str(event.key()) in arrow_dict:
+            hotkey = arrow_dict[str(event.key())]
+        else:
+            hotkey = str(event.text()).upper()
+        if hotkey in hotkey_dict:
+            hotkey_dict[hotkey]()
+
 
     def mousePressEvent(self, event):
-            if self.controller._dut:
-                marker = self._plot.markers[self._marker_tab.currentIndex()]
-                trace = self._plot.traces[marker.trace_index]
-                if event.button() == QtCore.Qt.MouseButton.LeftButton:
-                    click_pos =  event.pos().x() - 68
-                    plot_window_width = self._plot.window.width() - 68
+        if not self.controller._dut:
+            return
 
-                    if click_pos < plot_window_width and click_pos > 0:
-                        window_freq = self._plot.view_box.viewRange()[0]
-                        window_bw =  (window_freq[1] - window_freq[0])
-                        click_freq = ((float(click_pos) / float(plot_window_width)) * float(window_bw)) + window_freq[0]
-                        index = find_nearest_index(click_freq, trace.freq_range)
-                        self._plot.markers[self._marker_tab.currentIndex()].data_index = index
+        for marker in self._plot.markers:
+            if marker.selected:
+                break
+        else:
+            return
+
+        trace = self._plot.traces[marker.trace_index]
+        if event.button() == QtCore.Qt.MouseButton.LeftButton:
+            click_pos =  event.pos().x() - 68  # FIXME: declare this as a constant?
+            plot_window_width = self._plot.window.width() - 68
+
+            if click_pos < plot_window_width and click_pos > 0:
+                window_freq = self._plot.view_box.viewRange()[0]
+                window_bw =  (window_freq[1] - window_freq[0])
+                click_freq = ((float(click_pos) / float(plot_window_width)) * float(window_bw)) + window_freq[0]
+                index = find_nearest_index(click_freq, trace.freq_range)
+                marker.data_index = index
 
     def initUI(self):
-
-        self.init_menu()
         grid = QtGui.QGridLayout()
         grid.setSpacing(10)
         self.plot_width = 8
@@ -288,8 +386,8 @@ class MainPanel(QtGui.QWidget):
 
         controls_layout = QtGui.QVBoxLayout()
         controls_layout.addWidget(self._trace_controls())
-
-        controls_layout.addWidget(self._plot_controls())
+        self._plot_group = self.trace_group.plot_controls()
+        controls_layout.addWidget(self._plot_group)
         controls_layout.addWidget(self._device_controls())
         controls_layout.addWidget(self._freq_controls())
         controls_layout.addStretch()
@@ -298,59 +396,16 @@ class MainPanel(QtGui.QWidget):
         self._grid = grid
         self.setLayout(grid)
 
-    def init_menu(self):
-
-        # correct phase menu
-        cp_action = QtGui.QAction('&IQ Offset Correction', self)
-        cp_action.setCheckable(True)
-        cp_action.toggle()
-        self._dsp_menu.addAction(cp_action)
-        cp_action.triggered.connect(lambda: self.controller.apply_dsp_options(correct_phase=cp_action.isChecked()))
-
-        #dc offset correction
-        dc_action = QtGui.QAction('&DC Offset', self)
-        dc_action.setCheckable(True)
-        dc_action.toggle()
-        self._dsp_menu.addAction(dc_action)
-        dc_action.triggered.connect(lambda: self.controller.apply_dsp_options(hide_differential_dc_offset=dc_action.isChecked()))
-
-        #dbm conversion
-        dbm_action = QtGui.QAction('&Convert to dBm', self)
-        dbm_action.setCheckable(True)
-        dbm_action.toggle()
-        self._dsp_menu.addAction(dbm_action)
-        dbm_action.triggered.connect(lambda: self.controller.apply_dsp_options(convert_to_dbm=dbm_action.isChecked()))
-
-        # apply reference level
-        ref_Action = QtGui.QAction('&Apply Reference', self)
-        ref_Action.setCheckable(True)
-        ref_Action.toggle()
-        self._dsp_menu.addAction(ref_Action)
-        ref_Action.triggered.connect(lambda: self.controller.apply_dsp_options(apply_reference=ref_Action.isChecked()))
-
-        # apply spectral inversion
-        inv_action = QtGui.QAction('&Apply Spectral Inversion', self)
-        inv_action.setCheckable(True)
-        inv_action.toggle()
-        self._dsp_menu.addAction(inv_action)
-        inv_action.triggered.connect(lambda: self.controller.apply_dsp_options(apply_spec_inv=inv_action.isChecked()))
-
-        # apply hanning window
-        wind_action = QtGui.QAction('&Apply Hanning Window', self)
-        wind_action.setCheckable(True)
-        wind_action.toggle()
-        self._dsp_menu.addAction(wind_action)
-        wind_action.triggered.connect(lambda: self.controller.apply_dsp_options(apply_window=wind_action.isChecked()))
-
     def _plot_layout(self):
         vsplit = QtGui.QSplitter()
         vsplit.setOrientation(QtCore.Qt.Vertical)
         vsplit.addWidget(self._plot.window)
+        if self._plot.waterfall_window:
+            vsplit.addWidget(self._plot.waterfall_window)
 
         hsplit = QtGui.QSplitter()
         hsplit.addWidget(self._plot.const_window)
         hsplit.addWidget(self._plot.iq_window)
-        self._plot.const_window.heightForWidth(1)
         self._plot.const_window.hide()
         self._plot.iq_window.hide()
         vsplit.addWidget(hsplit)
@@ -359,13 +414,7 @@ class MainPanel(QtGui.QWidget):
         return self._plot_layout
 
     def _trace_controls(self):
-        self.trace_group = TraceControls()
-        self.trace_group.trace_attr['store'].clicked.connect(lambda: cu._store_trace(self))
-        self.trace_group.trace_attr['max_hold'].clicked.connect(lambda: cu.max_hold(self))
-        self.trace_group.trace_attr['min_hold'].clicked.connect(lambda: cu.min_hold(self))
-        self.trace_group.trace_attr['write'].clicked.connect(lambda: cu.trace_write(self))
-        self.trace_group.trace_attr['blank'].clicked.connect(lambda: cu.blank_trace(self))
-        self.trace_group.trace_tab.currentChanged.connect(lambda: cu._trace_tab_change(self))
+        self.trace_group = TraceControls(self.controller, self._plot)
         self.control_widgets.append(self.trace_group)
         return self.trace_group
 
@@ -384,102 +433,6 @@ class MainPanel(QtGui.QWidget):
         self.control_widgets.append(self._freq_group)
         return self._freq_group
 
-    def _plot_controls(self):
-
-        plot_group = QtGui.QGroupBox("Plot Control")
-        self._plot_group = plot_group
-        
-        plot_controls_layout = QtGui.QVBoxLayout()
-        
-        first_row = QtGui.QHBoxLayout()
-        marker_tab = QtGui.QTabBar()
-        for marker in labels.MARKERS:
-            marker_tab.addTab(marker)
-        marker_tab.currentChanged.connect(lambda: cu._marker_tab_change(self))
-        first_row.addWidget(marker_tab)
-        
-        self._marker_tab = marker_tab
-        self.control_widgets.append(self._marker_tab)
-        marker_check, marker_trace = self._marker_control()
-        
-        second_row = QtGui.QHBoxLayout()
-        second_row.addWidget(marker_trace)
-        second_row.addWidget(marker_check)
-                
-        third_row = QtGui.QHBoxLayout()
-        third_row.addWidget(self._peak_control())
-        third_row.addWidget(self._center_control())
-        
-        fourth_row = QtGui.QHBoxLayout()
-        ref_level, ref_label, min_level, min_label = self._ref_controls()
-        
-        fourth_row.addWidget(ref_label)
-        fourth_row.addWidget(ref_level)
-        fourth_row.addWidget(min_label)
-        fourth_row.addWidget(min_level)
-
-        plot_controls_layout.addLayout(first_row)
-        plot_controls_layout.addLayout(second_row)
-        plot_controls_layout.addLayout(third_row)
-        plot_controls_layout.addLayout(fourth_row)
-        plot_group.setLayout(plot_controls_layout)
-        
-        return plot_group
-        
-    def _marker_control(self):
-        marker_trace = QtGui.QComboBox()
-        marker_trace.setEnabled(False)
-        marker_trace.setMaximumWidth(50)
-        marker_trace.currentIndexChanged.connect(lambda: cu._marker_trace_control(self))
-        update_marker_traces(marker_trace, self._plot.traces)
-        
-        self._marker_trace = marker_trace
-        marker_check = QtGui.QCheckBox('Enabled')
-        marker_check.clicked.connect(lambda: cu._marker_control(self))
-        self._marker_check = marker_check
-
-        self.control_widgets.append(self._marker_check)
-        return marker_check,marker_trace
-            
-    def _peak_control(self):
-        peak = QtGui.QPushButton('Peak')
-        peak.setToolTip("[P]\nFind peak of the selected spectrum") 
-        peak.clicked.connect(lambda: cu._find_peak(self))
-        self._peak = peak
-        self.control_widgets.append(self._peak)
-        return peak
-                
-    def _center_control(self):
-        center = QtGui.QPushButton('Recenter')
-        center.setToolTip("[C]\nCenter the Plot View around the available spectrum") 
-        center.clicked.connect(lambda: self._plot.center_view(min(self.xdata), 
-                                                            max(self.xdata),
-                                                            min_level = int(self._min_level.text()),
-                                                            ref_level = int(self._ref_level.text())))
-        self._center_bt = center
-        self.control_widgets.append(self._center_bt)
-        return center
-    
-    def _ref_controls(self):
-        ref_level = QtGui.QLineEdit(str(PLOT_YMAX))
-        ref_level.returnPressed.connect(lambda: self._plot.center_view(min(self.xdata), 
-                                                                        max(self.xdata),
-                                                                        min_level = int(self._min_level.text()),
-                                                                        ref_level = int(self._ref_level.text())))
-        self._ref_level = ref_level
-        self.control_widgets.append(self._ref_level)
-        ref_label = QtGui.QLabel('Maximum Level: ')
-        
-        min_level = QtGui.QLineEdit(str(PLOT_YMIN)) 
-        min_level.returnPressed.connect(lambda: self._plot.center_view(min(self.xdata), 
-                                                                        max(self.xdata),
-                                                                        min_level = int(self._min_level.text()),
-                                                                        ref_level = int(self._ref_level.text())))
-        min_label = QtGui.QLabel('Minimum Level: ')
-        self._min_level = min_level
-        self.control_widgets.append(self._min_level)
-        return ref_level, ref_label, min_level, min_label
-
     def _marker_labels(self):
         marker_label = QtGui.QLabel('')
         marker_label.setStyleSheet('color: %s;' % colors.TEAL)
@@ -513,29 +466,63 @@ class MainPanel(QtGui.QWidget):
         self.xdata = np.linspace(fstart, fstop, len(power))
 
         self.update_trace()
-        if self.freq_range != (fstart, fstop):
-            self.freq_range = (fstart, fstop)
-            self._plot.center_view(fstart, fstop)
-        self.update_iq()
         self.update_marker()
         self.update_diff()
+        if not self.controller.applying_user_xrange():
+            self._plot.center_view(fstart, fstop)
+
+        if self.iq_plots_enabled:
+            self.update_iq()
+
+        if (fstart, fstop, len(power)) != self._waterfall_range:
+            self._plot.waterfall_data.reset(self.xdata)
+            self._waterfall_range = (fstart, fstop, len(power))
+        self._plot.waterfall_data.add_row(power)
+
+
+    def options_changed(self, options, changed):
+        self.iq_plots_enabled = options['iq_plots']
+        self.waterfall_plot_enabled = options['waterfall_plot']
+
+        if 'iq_plots' in changed:
+            self._update_iq_plot_visibility()
+
+        ww = self._plot.waterfall_window
+        if 'waterfall_plot' in changed and ww:
+            if options['waterfall_plot']:
+                ww.show()
+            else:
+                ww.hide()
+
+    def _update_iq_plot_visibility(self):
+        if not self.gui_state:
+            return
+        if self.gui_state.sweeping() or not self.iq_plots_enabled:
+            self._plot.const_window.hide()
+            self._plot.iq_window.hide()
+        else:
+            self._plot.const_window.show()
+            self._plot.iq_window.show()
+
+        ww = self._plot.waterfall_window
+        if self.waterfall_plot_enabled and ww:
+            ww.show()
+        else:
+            ww.hide()
 
     def update_trace(self):
-
-        #FIXME make alternate_colors user defined
         for trace in self._plot.traces:
             trace.update_curve(
                 self.xdata,
                 self.pow_data,
                 self.usable_bins,
-                self.sweep_segments,
-                self.plot_state.alt_colors)
+                self.sweep_segments)
 
     def update_iq(self):
 
         if not self.raw_data:
                 return
-        trace = self._plot.traces[self.trace_group.trace_tab.currentIndex()]
+        trace = self._plot.traces[0]
 
         if not (trace.write or trace.max_hold or trace.min_hold or trace.store):
             return
@@ -582,7 +569,7 @@ class MainPanel(QtGui.QWidget):
                         marker_label.setStyleSheet('color: rgb(%s, %s, %s);' % (trace.color[0],
                                                                              trace.color[1],
                                                                             trace.color[2]))
-                        
+
                         marker.update_pos(trace.freq_range, trace.data)
                         marker_text = 'Frequency: %0.2f MHz \n Power %0.2f dBm' % (trace.freq_range[marker.data_index]/1e6, 
                                                                                    trace.data[marker.data_index])
@@ -617,14 +604,7 @@ class MainPanel(QtGui.QWidget):
         for item in self.control_widgets:
             item.setEnabled(True)
 
-        for key in self.trace_group.trace_attr:
-            self.trace_group.trace_attr[key].setEnabled(True)
-        
     def disable_controls(self):
         for item in self.control_widgets:
             item.setEnabled(False)
 
-        for key in self.trace_group.trace_attr:
-            self.trace_group.trace_attr[key].setEnabled(False)
-
-        
